@@ -18,6 +18,7 @@
 """
 
 import threading
+import traceback
 import os.path
 
 
@@ -47,36 +48,42 @@ def main():
 
         try:
             import robotframework_debug_adapter
+            import robotframework_ls
         except ImportError:
             # Automatically add it to the path if __main__ is being executed.
             sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
             import robotframework_debug_adapter  # @UnusedImport
+            import robotframework_ls
+
+        robotframework_ls.import_robocode_ls_core()
 
         from robotframework_debug_adapter.debug_adapter_threads import (
             STOP_WRITER_THREAD,
         )
-        from robotframework_ls.robotframework_log import (
+        from robocode_ls_core.robotframework_log import (
             get_logger,
             configure_logger,
             log_args_and_python,
         )
-        import traceback
 
-        configure_logger("dap")
+        from robotframework_debug_adapter.constants import LOG_FILENAME
+        from robotframework_debug_adapter.constants import LOG_LEVEL
+
+        configure_logger("dap", LOG_LEVEL, LOG_FILENAME)
         log = get_logger("robotframework_debug_adapter.__main__")
-        log_args_and_python(log, sys.argv)
+        log_args_and_python(log, sys.argv, robotframework_ls.__version__)
 
         from robotframework_debug_adapter.debug_adapter_threads import reader_thread
         from robotframework_debug_adapter.debug_adapter_threads import writer_thread
-        from robotframework_debug_adapter.command_processor import CommandProcessor
+        from robotframework_debug_adapter.debug_adapter_comm import DebugAdapterComm
 
         try:
             from queue import Queue
         except ImportError:
             from Queue import Queue
 
-        write_queue = Queue()
-        command_processor = CommandProcessor(write_queue)
+        to_client_queue = Queue()
+        comm = DebugAdapterComm(to_client_queue)
 
         write_to = sys.stdout
         read_from = sys.stdin
@@ -95,12 +102,12 @@ def main():
 
         writer = threading.Thread(
             target=writer_thread,
-            args=(write_to, write_queue, "write to client"),
+            args=(write_to, to_client_queue, "write to client"),
             name="Write to client",
         )
         reader = threading.Thread(
             target=reader_thread,
-            args=(read_from, command_processor, write_queue, b"read from client"),
+            args=(read_from, comm.from_client, to_client_queue, b"read from client"),
             name="Read from client",
         )
 
@@ -109,7 +116,7 @@ def main():
 
         reader.join()
         log.debug("Exited reader.\n")
-        write_queue.put(STOP_WRITER_THREAD)
+        to_client_queue.put(STOP_WRITER_THREAD)
         writer.join()
         log.debug("Exited writer.\n")
     except:
