@@ -206,7 +206,7 @@ def find_variable(ast, line, col):
     token_info = find_token(ast, line, col)
     if token_info is not None:
         token = token_info.token
-        if token.type == token.ARGUMENT and "{" in token.value:
+        if "{" in token.value:
             for part in _tokenize_variables_even_when_invalid(token, col):
                 if part.col_offset <= col <= part.end_col_offset:
                     if part.type == part.VARIABLE:
@@ -218,11 +218,18 @@ def find_variable(ast, line, col):
     return None
 
 
+def create_token(name):
+    from robot.api import Token
+
+    return Token(Token.NAME, name)
+
+
+def tokenize_variables_from_name(name):
+    return tokenize_variables(create_token(name))  # May throw error if it's not OK.
+
+
 def tokenize_variables(token):
-    try:
-        return token.tokenize_variables()
-    except:  # If variables aren't correct, this may fail, so, return no variables.
-        return iter([])
+    return token.tokenize_variables()  # May throw error if it's not OK.
 
 
 def _tokenize_variables_even_when_invalid(token, col):
@@ -317,10 +324,18 @@ def iter_keyword_arguments_as_str(ast):
     """
     :rtype: generator(str)
     """
+    for token in iter_keyword_arguments_as_tokens(ast):
+        yield str(token)
+
+
+def iter_keyword_arguments_as_tokens(ast):
+    """
+    :rtype: generator(Token)
+    """
     for _stack, node in _iter_nodes_filtered(ast, accept_class="Arguments"):
         for token in node.tokens:
             if token.type == token.ARGUMENT:
-                yield str(token)
+                yield token
 
 
 def get_documentation(ast):
@@ -379,6 +394,8 @@ def iter_keyword_usage_tokens(ast):
 
 def get_keyword_name_token(ast, token):
     """
+    If the given token is a keyword, return the token, otherwise return None.
+    
     :note: this goes hand-in-hand with iter_keyword_usage_tokens.
     """
     from robocode_ls_core.basic import isinstance_name
@@ -390,6 +407,38 @@ def get_keyword_name_token(ast, token):
     return None
 
 
+def get_library_import_name_token(ast, token):
+    """
+    If the given ast node is a library import and the token is its name, return
+    the name token, otherwise, return None.
+    """
+    from robocode_ls_core.basic import isinstance_name
+
+    if (
+        token.type == token.NAME
+        and isinstance_name(ast, "LibraryImport")
+        and ast.name == token.value  # I.e.: match the name, not the alias.
+    ):
+        return token
+    return None
+
+
+def get_resource_import_name_token(ast, token):
+    """
+    If the given ast node is a library import and the token is its name, return
+    the name token, otherwise, return None.
+    """
+    from robocode_ls_core.basic import isinstance_name
+
+    if (
+        token.type == token.NAME
+        and isinstance_name(ast, "ResourceImport")
+        and ast.name == token.value  # I.e.: match the name, not the alias.
+    ):
+        return token
+    return None
+
+
 def _copy_of_node_replacing_token(node, token, token_type):
     """
     Workaround to create a new version of the same node but with the first
