@@ -1,12 +1,217 @@
 import os
 import sys
-from collections import namedtuple
 from robotframework_ls.constants import NULL
 from robocode_ls_core.robotframework_log import get_logger
+from robocode_ls_core.constants import IS_PY2
+from robocode_ls_core.basic import py2_filesystem_encode
 
 log = get_logger(__name__)
 
-_LibInfo = namedtuple("_LibInfo", "library_doc, mtime")
+
+def _normfile(filename):
+    return os.path.abspath(os.path.normpath(os.path.normcase(filename)))
+
+
+def _get_libspec_mutex_name(libspec_filename):
+    from robocode_ls_core.system_mutex import generate_mutex_name
+
+    libspec_filename = _norm_filename(libspec_filename)
+    basename = os.path.basename(libspec_filename)
+    name = os.path.splitext(basename)[0]
+    return generate_mutex_name(libspec_filename, prefix="%s_" % (name,))
+
+
+def _get_additional_info_filename(spec_filename):
+    additional_info_filename = os.path.join(spec_filename + ".m")
+    return additional_info_filename
+
+
+def _load_library_doc_and_mtime(spec_filename, obtain_mutex=True):
+    """
+    :param obtain_mutex:
+        Should be False if this is part of a bigger operation that already
+        has the spec_filename mutex.
+    """
+    from robotframework_ls.impl import robot_specbuilder
+    from robocode_ls_core.system_mutex import timed_acquire_mutex
+
+    if obtain_mutex:
+        ctx = timed_acquire_mutex(_get_libspec_mutex_name(spec_filename))
+    else:
+        ctx = NULL
+    with ctx:
+        # We must load it with a mutex to avoid conflicts between generating/reading.
+        builder = robot_specbuilder.SpecDocBuilder()
+        try:
+            mtime = os.path.getmtime(spec_filename)
+            libdoc = builder.build(spec_filename)
+            return libdoc, mtime
+        except Exception:
+            log.exception("Error when loading spec info from: %s", spec_filename)
+            return None
+
+
+def _load_lib_info(spec_filename, can_regenerate):
+    libdoc_and_mtime = _load_library_doc_and_mtime(spec_filename)
+    if libdoc_and_mtime is None:
+        return None
+    libdoc, mtime = libdoc_and_mtime
+    return _LibInfo(libdoc, mtime, spec_filename, can_regenerate)
+
+
+_IS_BUILTIN = "is_builtin"
+_SOURCE_TO_MTIME = "source_to_mtime"
+_UNABLE_TO_LOAD = "unable_to_load"
+
+
+def _create_updated_source_to_mtime(library_doc):
+    sources = set()
+
+    source = library_doc.source
+    if source is not None:
+        sources.add(source)
+
+    for keyword in library_doc.keywords:
+        source = keyword.source
+        if source is not None:
+            sources.add(source)
+
+    source_to_mtime = {}
+    for source in sources:
+        try:
+            source = _normfile(source)
+            source_to_mtime[source] = os.path.getmtime(source)
+        except Exception:
+            log.exception("Unable to load source for file: %s", source)
+    return source_to_mtime
+
+
+def _create_additional_info(spec_filename, is_builtin, obtain_mutex=True):
+    try:
+        additional_info = {_IS_BUILTIN: is_builtin}
+        if is_builtin:
+            # For builtins we don't have to check the mtime
+            # (on a new version we update the folder).
+            return additional_info
+
+        library_doc_and_mtime = _load_library_doc_and_mtime(
+            spec_filename, obtain_mutex=obtain_mutex
+        )
+        if library_doc_and_mtime is None:
+            additional_info[_UNABLE_TO_LOAD] = True
+            return additional_info
+
+        library_doc = library_doc_and_mtime[0]
+
+        additional_info[_SOURCE_TO_MTIME] = _create_updated_source_to_mtime(library_doc)
+        return additional_info
+
+    except:
+        log.exception(
+            "Error creating additional info for spec filename: %s", spec_filename
+        )
+        return {}
+
+
+def _load_spec_filename_additional_info(spec_filename):
+    """
+    Loads additional information given a spec filename.
+    """
+    import json
+
+    try:
+        additional_info_filename = _get_additional_info_filename(spec_filename)
+
+        with open(additional_info_filename, "r") as stream:
+            source_to_mtime = json.load(stream)
+        return source_to_mtime
+    except:
+        log.exception("Unable to load source mtimes from: %s", additional_info_filename)
+        return {}
+
+
+def _dump_spec_filename_additional_info(spec_filename, is_builtin, obtain_mutex=True):
+    """
+    Creates a filename with additional information not directly available in the
+    spec.
+    """
+    import json
+
+    source_to_mtime = _create_additional_info(
+        spec_filename, is_builtin, obtain_mutex=obtain_mutex
+    )
+    additional_info_filename = _get_additional_info_filename(spec_filename)
+    with open(additional_info_filename, "w") as stream:
+        json.dump(source_to_mtime, stream, indent=2, sort_keys=True)
+
+
+class _LibInfo(object):
+    __slots__ = [
+        "library_doc",
+        "mtime",
+        "_spec_filename",
+        "_additional_info",
+        "_invalid",
+        "_can_regenerate",
+    ]
+
+    def __init__(self, library_doc, mtime, spec_filename, can_regenerate):
+        """
+        :param library_doc:
+        :param mtime:
+        :param spec_filename:
+        :param bool can_regenerate:
+            False means that the information from this file can't really be
+            regenerated (i.e.: this is a spec file from a library or created
+            by the user).
+        """
+        assert library_doc
+        assert mtime
+        assert spec_filename
+
+        self.library_doc = library_doc
+        self.mtime = mtime
+
+        self._can_regenerate = can_regenerate
+        self._spec_filename = spec_filename
+        self._additional_info = None
+        self._invalid = False
+
+    def verify_sources_sync(self):
+        """
+        :return bool:
+            True if everything is ok and this library info can be used. Otherwise,
+            the spec file and the _LibInfo must be recreated. 
+        """
+        if not self._can_regenerate:
+            # This means that this info was generated by a library or the user
+            # himself, thus, we can't regenerate it.
+            return True
+
+        if self._invalid:  # Once invalid, always invalid.
+            return False
+
+        additional_info = self._additional_info
+        if additional_info is None:
+            additional_info = _load_spec_filename_additional_info(self._spec_filename)
+            if additional_info.get(_IS_BUILTIN, False):
+                return True
+
+            source_to_mtime = additional_info.get(_SOURCE_TO_MTIME)
+            if source_to_mtime is None:
+                # Nothing to validate...
+                return True
+
+            updated_source_to_mtime = _create_updated_source_to_mtime(self.library_doc)
+            if source_to_mtime != updated_source_to_mtime:
+                log.info(
+                    "Library %s is invalid. Current source to mtime:\n%s\nChanged from:\n%s"
+                    % (self.library_doc.name, source_to_mtime, updated_source_to_mtime)
+                )
+                self._invalid = True
+                return False
+
+        return True
 
 
 def _norm_filename(path):
@@ -38,8 +243,9 @@ class _FolderInfo(object):
             log.info("Tracking folder for changes: %s", self.folder_path)
             from robocode_ls_core.watchdog_wrapper import PathInfo
 
+            folder_path = self.folder_path
             self._watch = observer.notify_on_extensions_change(
-                [PathInfo(self.folder_path, recursive=self.recursive)],
+                [PathInfo(folder_path, recursive=self.recursive)],
                 ["libspec"],
                 notifier.on_change,
                 (self._on_change_spec,),
@@ -101,6 +307,8 @@ class _FolderInfo(object):
                     continue
                 else:
                     if info.mtime != curr_mtime:
+                        # The spec filename mtime changed, so, set to None
+                        # to reload it.
                         info = None
 
             new_libspec_filename_to_info[filename] = info
@@ -113,9 +321,9 @@ class LibspecManager(object):
     
     .libspec files are searched in the following directories:
 
-    - PYTHONPATH folders                              (not recursive)
-    - Workspace folders                               (recursive -- notifications from the LSP)
-    - ${user}robotframework_ls/lispec/${python_hash}  (not recursive)
+    - PYTHONPATH folders                                  (not recursive)
+    - Workspace folders                                   (recursive -- notifications from the LSP)
+    - ${user}.robotframework-ls/specs/${python_hash}      (not recursive)
 
     It searches for .libspec files in the folders tracked and provides the
     keywords that are available from those (properly caching data as needed).
@@ -142,8 +350,9 @@ class LibspecManager(object):
         except:
             v = "unknown"
 
+        # Note: _v1: information on the mtime of the libspec sources now available.
         return os.path.join(
-            user_home, "robotframework_ls", "libspec", "%s_%s" % (digest, v)
+            user_home, ".robotframework-ls", "specs", "%s_%s" % (digest, v)
         )
 
     @classmethod
@@ -307,24 +516,34 @@ class LibspecManager(object):
 
     def _gen_builtin_libraries(self):
         """
-        Generates .lispec files for the libraries builtin (if needed).
+        Generates .libspec files for the libraries builtin (if needed).
         """
         import time
 
         try:
             from robotframework_ls.impl import robot_constants
+            from robocode_ls_core.system_mutex import timed_acquire_mutex
+            from robocode_ls_core.system_mutex import generate_mutex_name
 
             initial_time = time.time()
-
             wait_for = []
-            for libname in robot_constants.STDLIBS:
-                library_info = self.get_library_info(libname, create=False)
-                if library_info is None:
-                    wait_for.append(
-                        self._thread_pool.submit(self._create_libspec, libname)
-                    )
-            for future in wait_for:
-                future.result()
+
+            with timed_acquire_mutex(
+                generate_mutex_name(
+                    _norm_filename(self._builtins_libspec_dir), prefix="gen_builtins_"
+                ),
+                timeout=100,
+            ):
+                for libname in robot_constants.STDLIBS:
+                    library_info = self.get_library_info(libname, create=False)
+                    if library_info is None:
+                        wait_for.append(
+                            self._thread_pool.submit(
+                                self._create_libspec, libname, is_builtin=True
+                            )
+                        )
+                for future in wait_for:
+                    future.result()
 
             if wait_for:
                 log.debug(
@@ -368,60 +587,61 @@ class LibspecManager(object):
         self.synchronize_additional_pythonpath_folders()
         self.synchronize_internal_libspec_folders()
 
-    def _iter_library_doc(self):
+    def _iter_lib_info(self):
         """
-        :rtype: generator(LibraryDoc)
+        :rtype: generator(_LibInfo)
         """
-        found_libraries = set()
-
         # Note: the iteration order is important (first ones are visited earlier
         # and have higher priority).
         iter_in = []
         for (_uri, info) in self._workspace_folder_uri_to_folder_info.items():
-            iter_in.append(info.libspec_filename_to_info)
+            iter_in.append((info.libspec_filename_to_info, False))
 
         for (_uri, info) in self._pythonpath_folder_to_folder_info.items():
-            iter_in.append(info.libspec_filename_to_info)
+            iter_in.append((info.libspec_filename_to_info, False))
 
         for (_uri, info) in self._additional_pythonpath_folder_to_folder_info.items():
-            iter_in.append(info.libspec_filename_to_info)
+            iter_in.append((info.libspec_filename_to_info, False))
 
         for (_uri, info) in self._internal_folder_to_folder_info.items():
-            iter_in.append(info.libspec_filename_to_info)
+            iter_in.append((info.libspec_filename_to_info, True))
+
+        for filename_to_info, can_regenerate in iter_in:
+            for spec_filename, info in list(filename_to_info.items()):
 
-        for filename_to_info in iter_in:
-            for filename, info in list(filename_to_info.items()):
-                if info is not None:
-                    try:
-                        if info.mtime != os.path.getmtime(filename):
-                            info = None
-                    except:
-                        continue
                 if info is None:
-                    info = filename_to_info[filename] = self._load_info(filename)
+                    info = filename_to_info[spec_filename] = _load_lib_info(
+                        spec_filename, can_regenerate
+                    )
 
-                if info is not None and info.library_doc is not None:
-                    if info.library_doc.name not in found_libraries:
-                        found_libraries.add(info.library_doc.name)
-                        yield info.library_doc
+                # Note: we could end up yielding a library with the same name
+                # multiple times due to its scope. It's up to the caller to
+                # validate that.
 
-    def _load_info(self, filename):
-        from robotframework_ls.impl import robot_specbuilder
-
-        builder = robot_specbuilder.SpecDocBuilder()
-        try:
-            mtime = os.path.getmtime(filename)
-            libdoc = builder.build(filename)
-            return _LibInfo(libdoc, mtime)
-        except Exception:
-            log.exception("Error when loading spec info from: %s", filename)
-            return None
+                # Note: we also check if there are keywords available... in
+                # some cases we may create libraries for namespace packages
+                # (i.e.: empty folders) which don't really have anything -- in
+                # this case, this isn't a valid library.
+                if (
+                    info is not None
+                    and info.library_doc is not None
+                    and info.library_doc.keywords
+                ):
+                    yield info
 
     def get_library_names(self):
-        return [library_doc.name for library_doc in self._iter_library_doc()]
+        return sorted(
+            set(lib_info.library_doc.name for lib_info in self._iter_lib_info())
+        )
 
     def _create_libspec(
-        self, libname, env=None, log_time=True, cwd=None, additional_path=None
+        self,
+        libname,
+        env=None,
+        log_time=True,
+        cwd=None,
+        additional_path=None,
+        is_builtin=False,
     ):
         """
         :param str libname:
@@ -430,68 +650,92 @@ class LibspecManager(object):
         import time
         from robotframework_ls.impl import robot_constants
         from robocode_ls_core.subprocess_wrapper import subprocess
+        from robocode_ls_core.system_mutex import timed_acquire_mutex
 
         curtime = time.time()
 
         try:
-            call = [sys.executable]
-            call.extend("-m robot.libdoc --format XML:HTML".split())
-            if additional_path:
-                if os.path.exists(additional_path):
-                    call.extend(["-P", additional_path])
-
-            for entry in list(self._additional_pythonpath_folder_to_folder_info.keys()):
-                if os.path.exists(entry):
-                    call.extend(["-P", entry])
-
-            call.append(libname)
-            libspec_dir = self._user_libspec_dir
-            if libname in robot_constants.STDLIBS:
-                libspec_dir = self._builtins_libspec_dir
-
-            target = os.path.join(libspec_dir, libname + ".libspec")
-            call.append(target)
-
-            mtime = -1
             try:
-                mtime = os.path.getmtime(target)
-            except:
-                pass
+                call = [sys.executable]
+                call.extend("-m robot.libdoc --format XML:HTML".split())
+                if additional_path:
+                    if os.path.exists(additional_path):
+                        call.extend(["-P", additional_path])
 
-            log.debug(
-                "Generating libspec for: %s.\nCommand line:\n%s",
-                libname,
-                " ".join(call),
-            )
-            try:
-                try:
-                    # Note: stdout is always subprocess.PIPE in this call.
-                    subprocess.check_output(
-                        call,
-                        stderr=subprocess.STDOUT,
-                        stdin=subprocess.PIPE,
-                        env=env,
-                        cwd=cwd,
-                    )
-                except OSError as e:
-                    log.exception("Error calling: %s", call)
-                    # We may have something as: Ignore OSError: [WinError 6] The handle is invalid,
-                    # give the result based on whether the file changed on disk.
+                additional_pythonpath_entries = list(
+                    self._additional_pythonpath_folder_to_folder_info.keys()
+                )
+                for entry in list(additional_pythonpath_entries):
+                    if os.path.exists(entry):
+                        call.extend(["-P", entry])
+
+                call.append(libname)
+                libspec_dir = self._user_libspec_dir
+                if libname in robot_constants.STDLIBS:
+                    libspec_dir = self._builtins_libspec_dir
+
+                libspec_filename = os.path.join(libspec_dir, libname + ".libspec")
+
+                with timed_acquire_mutex(
+                    _get_libspec_mutex_name(libspec_filename)
+                ):  # Could fail.
+                    call.append(libspec_filename)
+
+                    if IS_PY2:
+                        call = [py2_filesystem_encode(c) for c in call]
+
+                    mtime = -1
                     try:
-                        if mtime != os.path.getmtime(target):
-                            return True
+                        mtime = os.path.getmtime(libspec_filename)
                     except:
                         pass
 
-                    log.debug("Not retrying after OSError failure.")
-                    return False
+                    log.debug(
+                        "Generating libspec for: %s.\nCwd:%s\nCommand line:\n%s",
+                        libname,
+                        cwd,
+                        " ".join(call),
+                    )
+                    try:
+                        try:
+                            # Note: stdout is always subprocess.PIPE in this call.
+                            subprocess.check_output(
+                                call,
+                                stderr=subprocess.STDOUT,
+                                stdin=subprocess.PIPE,
+                                env=env,
+                                cwd=cwd,
+                            )
+                        except OSError as e:
+                            log.exception("Error calling: %s", call)
+                            # We may have something as: Ignore OSError: [WinError 6] The handle is invalid,
+                            # give the result based on whether the file changed on disk.
+                            try:
+                                if mtime != os.path.getmtime(libspec_filename):
+                                    _dump_spec_filename_additional_info(
+                                        libspec_filename,
+                                        is_builtin=is_builtin,
+                                        obtain_mutex=False,
+                                    )
+                                    return True
+                            except:
+                                pass
 
-            except subprocess.CalledProcessError as e:
-                log.exception(
-                    "Error creating libspec: %s. Output:\n%s", libname, e.output
-                )
+                            log.debug("Not retrying after OSError failure.")
+                            return False
+
+                    except subprocess.CalledProcessError as e:
+                        log.exception(
+                            "Error creating libspec: %s. Output:\n%s", libname, e.output
+                        )
+                        return False
+                    _dump_spec_filename_additional_info(
+                        libspec_filename, is_builtin=is_builtin, obtain_mutex=False
+                    )
+                    return True
+            except Exception:
+                log.exception("Error creating libspec: %s", libname)
                 return False
-            return True
         finally:
             if log_time:
                 delta = time.time() - curtime
@@ -501,6 +745,36 @@ class LibspecManager(object):
         self._observer.dispose()
         self._spec_changes_notifier.dispose()
 
+    def _do_create_libspec_on_get(self, libname, current_doc_uri):
+        from robocode_ls_core import uris
+
+        additional_path = None
+        abspath = None
+        cwd = None
+        if current_doc_uri is not None:
+            cwd = os.path.dirname(uris.to_fs_path(current_doc_uri))
+            if not cwd or not os.path.isdir(cwd):
+                cwd = None
+
+        if os.path.isabs(libname):
+            abspath = libname
+
+        elif current_doc_uri is not None:
+            # relative path: let's make it absolute
+            fs_path = os.path.dirname(uris.to_fs_path(current_doc_uri))
+            abspath = os.path.abspath(os.path.join(fs_path, libname))
+
+        if abspath:
+            additional_path = os.path.dirname(abspath)
+            libname = os.path.basename(libname)
+            if libname.lower().endswith((".py", ".class", ".java")):
+                libname = os.path.splitext(libname)[0]
+
+        if self._create_libspec(libname, additional_path=additional_path, cwd=cwd):
+            self.synchronize_internal_libspec_folders()
+            return True
+        return False
+
     def get_library_info(self, libname, create=True, current_doc_uri=None):
         """
         :param libname:
@@ -509,8 +783,8 @@ class LibspecManager(object):
 
         :rtype: LibraryDoc
         """
-        from robocode_ls_core import uris
-
+        if IS_PY2:
+            libname = py2_filesystem_encode(libname)
         libname_lower = libname.lower()
         if libname_lower.endswith((".py", ".class", ".java")):
             libname_lower = os.path.splitext(libname)[0]
@@ -518,35 +792,29 @@ class LibspecManager(object):
         if "/" in libname_lower or "\\" in libname_lower:
             libname_lower = os.path.basename(libname_lower)
 
-        for library_doc in self._iter_library_doc():
+        for lib_info in self._iter_lib_info():
+            library_doc = lib_info.library_doc
             if library_doc.name and library_doc.name.lower() == libname_lower:
-                return library_doc
+                if not lib_info.verify_sources_sync():
+                    if create:
+                        # Found but it's not in sync. Try to regenerate (don't proceed
+                        # because we don't want to match a lower priority item, so,
+                        # regenerate and get from the cache without creating).
+                        self._do_create_libspec_on_get(libname, current_doc_uri)
+
+                        # Note: get even if it if was not created (we may match
+                        # a lower priority library).
+                        return self.get_library_info(
+                            libname, create=False, current_doc_uri=current_doc_uri
+                        )
+                    else:
+                        # Not in sync and it should not be created, just skip it.
+                        continue
+                else:
+                    return library_doc
 
         if create:
-            additional_path = None
-            abspath = None
-            cwd = None
-            if current_doc_uri is not None:
-                cwd = os.path.dirname(uris.to_fs_path(current_doc_uri))
-                if not cwd or not os.path.isdir(cwd):
-                    cwd = None
-
-            if os.path.isabs(libname):
-                abspath = libname
-
-            elif current_doc_uri is not None:
-                # relative path: let's make it absolute
-                fs_path = os.path.dirname(uris.to_fs_path(current_doc_uri))
-                abspath = os.path.abspath(os.path.join(fs_path, libname))
-
-            if abspath:
-                additional_path = os.path.dirname(abspath)
-                libname = os.path.basename(libname)
-                if libname.lower().endswith((".py", ".class", ".java")):
-                    libname = os.path.splitext(libname)[0]
-
-            if self._create_libspec(libname, additional_path=additional_path, cwd=cwd):
-                self.synchronize_internal_libspec_folders()
+            if self._do_create_libspec_on_get(libname, current_doc_uri):
                 return self.get_library_info(
                     libname, create=False, current_doc_uri=current_doc_uri
                 )
