@@ -6,6 +6,31 @@ BDD_PREFIXES = ["given ", "when ", "then ", "and ", "but "]
 VARIABLE_PREFIXES = ("@", "%", "$", "&")
 
 
+# From: robot.variables.scopes.GlobalVariables._set_built_in_variables
+# (var name and description)
+BUILTIN_VARIABLES = [
+    ("${TEMPDIR}", "abspath(tempfile.gettempdir())"),
+    ("${EXECDIR}", "abspath('.')"),
+    ("${/}", "os.sep"),
+    ("${:}", "os.pathsep"),
+    ("${\\n}", "os.linesep"),
+    ("${SPACE}", " "),
+    ("${True}", "True"),
+    ("${False}", "False"),
+    ("${None}", "None"),
+    ("${null}", "None"),
+    ("${OUTPUT_DIR}", "settings.output_directory"),
+    ("${OUTPUT_FILE}", "settings.output or 'NONE'"),
+    ("${REPORT_FILE}", "settings.report or 'NONE'"),
+    ("${LOG_FILE}", "settings.log or 'NONE'"),
+    ("${DEBUG_FILE}", "settings.debug_file or 'NONE'"),
+    ("${LOG_LEVEL}", "settings.log_level"),
+    ("${PREV_TEST_NAME}", ""),
+    ("${PREV_TEST_STATUS}", ""),
+    ("${PREV_TEST_MESSAGE}", ""),
+]
+
+
 class Section(object):
 
     # Header to the section (i.e.: *** Setting ***)
