@@ -1,4 +1,6 @@
 from robocode_ls_core.client_base import LanguageServerClientBase
+from robocode_ls_core.constants import IS_PY2
+from robocode_ls_core.basic import py2_filesystem_decode
 
 
 class SubprocessDiedError(Exception):
@@ -24,10 +26,13 @@ class RobotFrameworkApiClient(LanguageServerClientBase):
         return True
 
     def initialize(
-        self, msg_id=None, process_id=None, root_uri="", workspace_folders=()
+        self, msg_id=None, process_id=None, root_uri=u"", workspace_folders=()
     ):
         from robocode_ls_core.options import NO_TIMEOUT, USE_TIMEOUTS
 
+        if IS_PY2:
+            root_uri = py2_filesystem_decode(root_uri)
+
         self._check_process_alive()
         msg_id = msg_id if msg_id is not None else self.next_id()
         return self.request(
@@ -83,6 +88,16 @@ class RobotFrameworkApiClient(LanguageServerClientBase):
             {"jsonrpc": "2.0", "id": msg_id, "method": method_name, "params": params}
         )
 
+    def forward_async(self, method_name, params):
+        """
+        :Note: async complete (returns _MessageMatcher).
+        """
+        self._check_process_alive()
+        msg_id = self.next_id()
+        return self.request_async(
+            {"jsonrpc": "2.0", "id": msg_id, "method": method_name, "params": params}
+        )
+
     def open(self, uri, version, source):
         self.forward(
             "textDocument/didOpen",
